/*******************************************************************************
modulo: 					TXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						15/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ 
							Una stringa composta da 8 caratteri viene inviata al modulo ricevente. Entrambe le stringhe,
							quella trasmessa e quella ricevuta, vengono visualizzate sui display lcd della freedom II
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE
							RE2 = CSN
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)
							
							RA1 = Potenziometro 10k

							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//************************************************************
// INCLUDE
//************************************************************
#include <xc.h>
#include <LTlib.h>
#include "LTlib_delay.h"
#include "LTlib_delay.c"
#include <module_IO.h>
#include <module_IO.c>
#include <module_SPI.h>
#include <module_SPI.c>
#include <LCD_44780.h>
#include <LCD_44780.c>
#include "nrf24l01.h"

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************	
int main(void) 
{
	// Variabile utilizzata per il numero delle letture
	unsigned char i;
	
	// Array per i dati da trasmettere
	unsigned char bufferTX[] = "ACBE1234";
	
	// Imposto tutte le porte come ingressi - I/O
    IO_set_all_ports_as_inputs();
	
	// Imposto tutta la PORTE come uscita per i segnali CE e CSN - I/O
	IO_set_port_direction(IO_PORTE, IO_ALL_PORT_OUTPUT);
  
	// Inizializzo il display LCD
	LCD_initialize(20);
		
	// Imposto tutte le porte analogiche come digitali - I/O
	ADCON1 = 0x0F;
		
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;

	 // Apro l'SPI        
    SPI1_baudrate(SPI_CLK_OSC_16);
    SPI1_mode (SPI_MODE_0);
    SPI1_open (SPI_MASTER_DEVICE);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_TX();
	
	while(1)
		{
			// Imposto il display lcd
			LCD_clear ();
			LCD_write_string (bufferTX);
			
			delay_ms(50);
			
						
					// Pulisco i buffer TX e RX
					Clear_BUFFER_TX_RX();
					
					// Ripulisco il registro STATUS
					Clear_STATUS_reg();
					
					// Scrivo il buffer TX
					unsigned char i;
					NRF24L01_CSN = 0;                                    
					SPI1_write_byte(W_TX_PAYLOAD);                       
					for(i = 0; i < PAYLOAD_BYTES; i++)     
					   {
						SPI1_write_byte (bufferTX[i]);
					   }
					NRF24L01_CSN = 1;  
					NRF24L01_CE = 1;
					delay_ms(1);
					NRF24L01_CE = 0;
					delay_ms(100);
		}
}