/*******************************************************************************
modulo: 					RXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						14/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ con ADC (collegare un potenziometro ad AN1)
							La lettura adc inviata (fino a 1023) e quella ricevuta vengono mostrate entrambe su due display lcd
picmicro: 					PIC 18F4550
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21
libreria Laurtec:			LTlib v4.0.4

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE			(si può usare qualunque altro pin)
							RE2 = CSN			(si può usare qualunque altro pin)
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)

							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//*************************************
// INCLUDE
//*************************************
#include <xc.h>
#include <LTlib.h>
#include "LTlib_delay.h"
#include "LTlib_delay.c"
#include <module_IO.h>
#include <module_IO.c>
#include <module_SPI.h>
#include <module_SPI.c>
#include <LCD_44780.h>
#include <LCD_44780.c>
#include "nrf24l01.h"

//*************************************
// PROGRAMMA PRINCIPALE RX
//*************************************
int main(void) 
{
	// Array per i dati da ricevere
	unsigned char bufferRX[32];
	
	// Variabile che contiene il valore adc totale (CASTING da Char a Int per i valori contenuti nei registri ADRSEL e ADRSEH)
	unsigned int sommatoria;

	// Imposto tutte le porte come ingressi - I/O
    IO_set_all_ports_as_inputs();
	
	// Imposto tutta la PORTE come uscita per i segnali CE e CSN - I/O
	IO_set_port_direction(IO_PORTE, IO_ALL_PORT_OUTPUT);
  
	// Inizializzo il display LCD
	LCD_initialize(20);
		
	// Imposto tutte le porte analogiche come digitali - I/O
	ADCON1 = 0x0F;
	
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;

    // Apro l'SPI        
    SPI1_baudrate(SPI_CLK_OSC_16);
    SPI1_mode (SPI_MODE_0);
    SPI1_open (SPI_MASTER_DEVICE);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_RX(); 
	
	while(1)
		{
			// Inizializzo la variabile
			sommatoria = 0;
			
			// Attendo i dati in arrivo
			while(NRF24L01_DataReady()){}
			
			// Leggo i dati ricevuti
			NRF24L01_CSN = 0;                                    
			SPI1_write_byte(R_RX_PAYLOAD); 
			bufferRX[0] = SPI1_read_byte();	// ADRSEL
			bufferRX[1] = SPI1_read_byte();	// ADRSEH
			
			// Ricostruisco il valore adc trasmesso. (Ogni valore viene convertito da Char a Int)
			sommatoria = (bufferRX[1] << 8) | bufferRX[0];
			
			// Visualizzo sul display, come stringa, il valore adc trasmesso
			LCD_home();
			LCD_write_message("ADC: ");
			LCD_goto_xy (8,1);
			LCD_write_integer (sommatoria, 4, ZERO_CLEANING_ON);
			
			// Imposto il puntatore su indirizzo 0 (terminatore di stringa)
			//bufferRX[0] = NULL;
			//bufferRX[1] = NULL;
			NRF24L01_CSN = 1;
						
			// Puliaco STATUS.
			Clear_STATUS_reg();
			
			// Pulisco buffers
			Clear_BUFFER_TX_RX(); 
	    }
}