/*******************************************************************************
modulo: 					TXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						14/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ con ADC (collegare un potenziometro ad AN1)
							La lettura adc inviata (fino a 1023) e quella ricevuta vengono mostrate entrambe su due display lcd
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21
libreria Laurtec:			LTlib v4.0.4

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE			(si può usare qualunque altro pin)
							RE2 = CSN			(si può usare qualunque altro pin)
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)

							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//*************************************
// INCLUDE
//*************************************
#include <xc.h>
#include <LTlib.h>
#include "LTlib_delay.h"
#include "LTlib_delay.c"
#include <module_IO.h>
#include <module_IO.c>
#include <module_ADC.h>
#include <module_ADC.c>
#include <module_SPI.h>
#include <module_SPI.c>
#include <LCD_44780.h>
#include <LCD_44780.c>
#include "nrf24l01.h"

//*************************************
// PROGRAMMA PRINCIPALE TX
//*************************************
int main(void) 
{
	// Array per memorizzare i dati da trasmettere
	unsigned char bufferTX[32];

	// Imposto tutte le porte come ingressi - I/O
    IO_set_all_ports_as_inputs();
	
	// Imposto tutta la PORTE come uscita per i segnali CE e CSN - I/O
	IO_set_port_direction(IO_PORTE, IO_ALL_PORT_OUTPUT);
  
	// Inizializzo il display LCD
	LCD_initialize(20);
		
	// Imposto tutte le porte analogiche come digitali - I/O
	ADCON1 = 0x0F;
	
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;

    // Apro l'SPI        
    SPI1_baudrate(SPI_CLK_OSC_16);
    SPI1_mode (SPI_MODE_0);
    SPI1_open (SPI_MASTER_DEVICE);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_TX(); 
   
    while (1) 
	{
			LCD_home();
			LCD_write_message("ADC: ");
			LCD_write_integer (ADC_read_channel(ADC_CH1), 4, LCD_ZERO_CLEANING_ON);     
			
			delay_ms(50);
			
			// L'adc del pic 18F4550 ha una risoluzione pari a 10 bit (0-1023), il che significa che ogni suo valore intero è composto da due 
			// byte, uno basso ed uno alto, memorizzati nei registri ADRESL e ADRESH. Poichè il payload (pacchetto di dati) minimo che posso 
			// inviare è un byte (max 32), per ricomporre questo intero devo disporre di due byte, sia in TX che in RX, ognuno dei quali mi 
			// serve per inviare i due valori separatamente. Poichè a tutti gli effetti ogni byte rappresenta un carattere da 0 a 255, faccio 
			// un casting da Int a Char per ognuno di essi quando li trasmetto. In fase di ricezione farò la cosa contraria.......
			// CASTING da INT a CHAR per i valori low e high contenuti nei registri ADC_BUFFER_LOW e ADC_BUFFER_HIGH (corrispondenti ai registri
			// ADRESL e ADRESH)
			bufferTX[0] = (unsigned char) (ADC_BUFFER_LOW);
			bufferTX[1] = (unsigned char) (ADC_BUFFER_HIGH);
				
					// Pulisco i buffer TX e RX
					Clear_BUFFER_TX_RX();
					
					// Ripulisco il registro STATUS
					Clear_STATUS_reg();
					
					// Scrivo il buffer TX
					unsigned char i;
					NRF24L01_CSN = 0;                                    
					SPI1_write_byte (W_TX_PAYLOAD);                       
					SPI1_write_byte  (bufferTX[0]);
					SPI1_write_byte  (bufferTX[1]);
					NRF24L01_CSN = 1;  
					NRF24L01_CE = 1;
					delay_ms(1);
					NRF24L01_CE = 0;
					delay_ms(100);
			
				
			
				
	}
        
}

