/*******************************************************************************

Author : Mauro Laurenti
Version : 1.7

Created on Date : 04/09/2006
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
									

#ifdef __XC8
#include <xc.h>
#include <stdlib.h>
#endif

#include "LCD_44780.h"


//************************************************************
//                  enable_pulse_LCD  Implementation
//************************************************************
void LCD_enable_pulse (void) {
	
	LCD_E = 1;
	delay_ms (1);
	LCD_E = 0;
	delay_ms (1);
}


//************************************************************
//                  send_command_LCD  Implementation
//************************************************************
void LCD_send_command (unsigned char D3, unsigned char D2, unsigned char D1, unsigned char D0) {

	LCD_D0 = D0;
	LCD_D1 = D1;
	LCD_D2 = D2;
	LCD_D3 = D3;
	enable_pulse_LCD ();
}


//************************************************************
//                  home_LCD  Implementation
//************************************************************
void LCD_home(void) {
	
	send_command_LCD (0,0,0,0);
	send_command_LCD (0,0,1,0);
}


//************************************************************
//                  shift_LCD  Implementation
//************************************************************
void LCD_shift (unsigned char shift, unsigned char number_of_shift) {
	
	unsigned char i;
	
	for (i=0; i < number_of_shift; i++) {	
		send_command_LCD (0,0,0,1);
		send_command_LCD (1,shift,0,0);
	}	
}


//************************************************************
//                  shift_cursor_LCD  Implementation
//************************************************************
void LCD_shift_cursor (unsigned char shift, unsigned char number_of_shift){
	
	unsigned char i;
	
	for (i=0; i < number_of_shift; i++) {
		send_command_LCD (0,0,0,1);
		send_command_LCD (0,shift,0,0);
	} 		
}

//************************************************************
//                  goto_line_LCD  Implementation
//************************************************************
void LCD_goto_line (unsigned char line) {

switch(line) {

	case 1: send_command_LCD(1,0,0,0);
		send_command_LCD(0,0,0,0);
			break;

	case 2: send_command_LCD(1,1,0,0);
		send_command_LCD(0,0,0,0);
			break;

	case 3: send_command_LCD(1,0,0,1);
		send_command_LCD(0,1,0,0);
			break;

	case 4: send_command_LCD(1,1,0,1);
                send_command_LCD(0,1,0,0);
	}
}


//************************************************************
//                  goto_xy_LCD  Implementation
//************************************************************
void LCD_goto_xy (unsigned char x, unsigned char y){

    goto_line_LCD (y);
    shift_cursor_LCD (RIGHT, x-1);
}

//************************************************************
//                  write_char_LCD  Implementation
//************************************************************
void LCD_write_char (unsigned char value) {
	
	unsigned char D3,D2,D1,D0;
	
	LCD_RS = 1;
		
	// Splitting of the first nibble				
	D3 = (value & 0b10000000) >> 7;	
	D2 = (value & 0b01000000) >> 6;
	D1 = (value & 0b00100000) >> 5;
	D0 = (value & 0b00010000) >> 4;
		
	send_command_LCD (D3,D2,D1,D0);
		
	// Splitting of the second nibble
	D3 = (value & 0b00001000) >> 3;	
	D2 = (value & 0b00000100) >> 2;
	D1 = (value & 0b00000010) >> 1;
	D0 = (value & 0b00000001);
			
	send_command_LCD (D3,D2,D1,D0);
		
	LCD_RS = 0;
}

//************************************************************
//                  write_message_LCD  Implementation
//************************************************************
#ifndef __XC8
void LCD_write_message (const rom unsigned char *buffer) {
#endif

#ifdef __XC8
void LCD_write_message (const unsigned char *buffer) {
#endif
	 // Write data to LCD up to null
    while(*buffer) {
	    
	    // Write character to LCD
	    write_char_LCD(*buffer);
	    // Increment buffer
	    buffer++;              
   }
}


//************************************************************
//                  write_string_LCD  Implementation
//************************************************************
void LCD_write_string (unsigned char *buffer) {
	
	// Write data to LCD up to null
	while(*buffer){
		
		// Write character to LCD
		write_char_LCD(*buffer);
		// Increment buffer
		buffer++;               
	}
}


//************************************************************
//                  write_integer_LCD  Implementation
//************************************************************
void LCD_write_integer (signed int value, unsigned char number_of_digits, unsigned char zero_cleaning){
	
	// The array size is 5 plus end of string \0
	unsigned char convertedInt [6] = {0,0,0,0,0,0};
	
	// Index used to shift to the right the digit
	unsigned char index;
	
	// Integer is converted to string
        #ifndef __XC8
            itoa (value, (unsigned char*) convertedInt);
        #endif

        #ifdef __XC8
            itoa ((unsigned char*) convertedInt, value,10);
	#endif


	if (number_of_digits >0 ) {
		
		convertedInt[number_of_digits] = '\0';
		
		// Shift the digit to the right removing the empty one
		//while (!isdigit(convertedInt[number_of_digits-1]))
                while (!(convertedInt[number_of_digits-1] <= '9' && convertedInt[number_of_digits-1] >= '0')){
			
			for (index = number_of_digits-1; index > 0; index--){
				convertedInt[index] = convertedInt[index-1];
                                
                                if (zero_cleaning == ZERO_CLEANING_ON) {
                                    convertedInt[index-1] = ' ';
                                } else {
                                    convertedInt[index-1] = '0';
                                }
				
			}							
		}	
	}

	write_string_LCD (convertedInt);
	
}	


//************************************************************
//                  clear_LCD  Implementation
//************************************************************
void LCD_clear (void){
	
	send_command_LCD (0,0,0,0);
	send_command_LCD (0,0,0,1);
}

//************************************************************
//                  cursor_LCD  Implementation
//************************************************************
void LCD_cursor (unsigned char active, unsigned char blinking) {
	
	send_command_LCD (0,0,0,0);
	send_command_LCD (1,1,active,blinking);
}

//************************************************************
//                  back_light_LCD  Implementation
//************************************************************
void LCD_backlight (unsigned char active) {

	LCD_LED = active;	
}

//************************************************************
//                  initialize_LCD  Implementation
//************************************************************
void  LCD_initialize (unsigned char quartz_frequency) {
	
	delay_set_quartz (quartz_frequency);
	
	LCD_RS = 0x00;
	LCD_E = 0x00;
	LCD_RW = 0x00;

	delay_ms (100);
	send_command_LCD (0,0,1,1);
	delay_ms (100);
	send_command_LCD (0,0,1,1);
	delay_ms (10);
	send_command_LCD (0,0,1,1);

	send_command_LCD (0,0,1,0);

        send_command_LCD (0,0,1,0);
	send_command_LCD (1,0,0,0);

	send_command_LCD (0,0,0,0);
        send_command_LCD (1,0,0,0);

	send_command_LCD (0,0,0,0);
	send_command_LCD (0,0,0,1);
        
        send_command_LCD (0,0,0,0);
	send_command_LCD (0,1,1,0);

	clear_LCD ();
       
        cursor_LCD (0,0);

}

