
/*******************************************************************************
modulo: 					RXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						17/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ con ADC e LCD (collegare un potenziometro ad AN1)
							La lettura adc inviata (fino a 1023) e quella ricevuta vengono mostrate entrambe su due display lcd
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE
							RE2 = CSN
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)
							
							RA1 = Potenziometro 10k

							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//************************************************************
// INCLUDE
//************************************************************
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <pic18f4550.h>
#include "PIC18F4550_config.h"
#include <spi.h>
#include "nrf24l01.h"
#include "LCD_44780.h"
#include "LCD_44780.c"
#include "delay.h"

//************************************************************
// DEFINE
//************************************************************
#define LINE_SPI_SCK   LATBbits.LATB1         
#define LINE_SPI_SDO   LATCbits.LATC7
#define LINE_SPI_SDI   LATBbits.LATB0
#define LINE_SPI_SS	   LATAbits.LATA5	// non usata

#define LCD_DEFAULT

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************	
int main(void) 
{
	// Array per i dati da ricevere
	unsigned char bufferRX[32];
	
	// Variabile che contiene il valore adc totale (CASTING da Char a Int per i valori contenuti nei registri ADRSEL e ADRSEH)
	unsigned int sommatoria;
    
    //Imposto PORTA
	LATA = 0x00;
	TRISA = 0b11011111;		// RA5 impostato in uscita per linea SS
	
	// Imposto PORTB
	LATB = 0x00;
	TRISB = 0b11111101;		// RB0 impostato in ingresso per linea SDI (MISO) - RB1 impostato in uscita per linea SCK (CLOCK)
	
	// Imposto PORTC
	LATC = 0x00;
	TRISC = 0b01111111;		// RC7 impostato in uscita per linea (MOSI)
	
	// Imposto PORTD
	LATD = 0x00;
	TRISD = 0x00;			// Tutta in uscita per pilotaggio display lcd
	
	// Imposto PORTE
	LATE = 0x00;
	TRISE = 0x00;			// RE0 impostato in uscita per linea CE - RE2 impostato in uscita per linea CSN 
	
	 // Imposto LCD
    LCD_initialize (20);
	
	// Imposto il display lcd
	LCD_clear ();
	LCD_write_message ("ADC = ");
		
	// Imposto tutte le porte analogiche come digitali - I/O
	ADCON1 = 0x0F;
	
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;
	
	// Inizializzo le linee per l'SPI
    LINE_SPI_SCK = 0x00;
    LINE_SPI_SDO = 0x00;
    LINE_SPI_SDI = 0x01;

	// Chiudo l'SPI per evitare eventuali errori)
	CloseSPI() ;
	
	//Inizializzazione del modulo SPI
	//Baudrate Fosc/16 @20MHz -> 1250000 KHz
	OpenSPI(SPI_FOSC_16,MODE_00,SMPEND);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_RX(); 
	
	while(1)
		{
			// Inizializzo la variabile
			sommatoria = 0;
			
			// Attendo i dati in arrivo
			while(NRF24L01_DataReady()){}
			
			// Leggo i dati ricevuti
			NRF24L01_CSN = 0;                                    
			WriteSPI(R_RX_PAYLOAD); 
			bufferRX[0] = ReadSPI();	// ADRSEL
			bufferRX[1] = ReadSPI();	// ADRSEH
			
			// Ricostruisco il valore adc trasmesso. (Ogni valore viene convertito da Char a Int)
			sommatoria = (bufferRX[1] << 8) | bufferRX[0];
			
			// Visualizzo sul display, come stringa, il valore adc trasmesso
			LCD_goto_xy (8,1);
			LCD_write_integer (sommatoria, 4, ZERO_CLEANING_ON);
			
			// Imposto il puntatore su indirizzo 0 (terminatore di stringa)
			bufferRX[0] = NULL;
			bufferRX[1] = NULL;
			NRF24L01_CSN = 1;
			
			// Ripulisco il registro STATUS
			Clear_STATUS_reg ();

			// Pulisco i buffer TX e RX
			Clear_BUFFER_TX_RX ();
	    }
}