/*******************************************************************************

Author : Mauro Laurenti
Version : 1.7

Created on Date : 04/09/2006
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller.							
 	
 To use these functions for your purpose you must set up properly 
 the LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

*******************************************************************************/

#ifndef LCD_44780_H
#define LCD_44780_H

#ifdef __XC8
    #include <xc.h>
    #include <stdlib.h>
#endif

#include "delay.h"


#ifndef __XC8
    #include <ctype.h>
#endif

//**************************************************
// LCD constants
// All the following pins must be set as output
//**************************************************

/**
 * This value are used as default ones
 *
 */
#ifndef LCD_DEFAULT

        #ifdef _PIC18
            #warning LCD_D0 has been not defined, LATDbits.LATD4 will be used
            #warning LCD_D1 has been not defined, LATDbits.LATD5 will be used
            #warning LCD_D2 has been not defined, LATDbits.LATD6 will be used
            #warning LCD_D3 has been not defined, LATDbits.LATD7 will be used
            #warning LCD_RS has been not defined, LATDbits.LATD2 will be used
            #warning LCD_E has been not defined, LATDbits.LATD3 will be used
            #warning LCD_RW has been not defined, LATDbits.LATD1 will be used
            #warning LCD_LED has been not defined, LATCbits.LATC1 will be used
        #endif

        #ifndef _PIC18
            #warning LCD_D0 has been not defined, PORTDbits.RD4 will be used
            #warning LCD_D1 has been not defined, PORTDbits.RD5 will be used
            #warning LCD_D2 has been not defined, PORTDbits.RD6 will be used
            #warning LCD_D3 has been not defined, PORTDbits.RD7 will be used
            #warning LCD_RS has been not defined, PORTDbits.RD2 will be used
            #warning LCD_E has been not defined, PORTDbits.RD3 will be used
            #warning LCD_RW has been not defined, PORTDbits.RD1 will be used
            #warning LCD_LED has been not defined, PORTCbits.RC1 will be used
        #endif
	
#endif
        #ifdef _PIC18
            #define LCD_D0 LATDbits.LATD4
            #define LCD_D1 LATDbits.LATD5
            #define LCD_D2 LATDbits.LATD6
            #define LCD_D3 LATDbits.LATD7
            #define LCD_RS LATDbits.LATD2
            #define LCD_E  LATDbits.LATD3
            #define LCD_RW LATDbits.LATD1
            #define LCD_LED LATCbits.LATC1
        #endif

        #ifndef _PIC18
            #define LCD_D0 PORTDbits.RD4
            #define LCD_D1 PORTDbits.RD5
            #define LCD_D2 PORTDbits.RD6
            #define LCD_D3 PORTDbits.RD7
            #define LCD_RS PORTDbits.RD2
            #define LCD_E  PORTDbits.RD3
            #define LCD_RW PORTDbits.RD1
            #define LCD_LED PORTCbits.RC1
        #endif
	


//**************************************************
//               Constant Definitions
//**************************************************

#define LEFT 0
#define RIGHT 1
#define LCD_LEFT 0
#define LCD_RIGHT 1

#define TURN_ON_LED_LCD 1
#define TURN_OFF_LED_LCD 0
#define LCD_TURN_ON_LED 1
#define LCD_TURN_OFF_LED 0

#define LCD_TURN_ON_LED 1
#define LCD_TURN_OFF_LED 0


#define TURN_ON_CURSOR 1
#define TURN_OFF_CURSOR 0
#define LCD_TURN_ON_CURSOR 1
#define LCD_TURN_OFF_CURSOR 0

#define BLINKING_ON 1
#define BLINKING_OFF 0
#define LCD_BLINKING_ON 1
#define LCD_BLINKING_OFF 0

#define ZERO_CLEANING_ON 0x01
#define ZERO_CLEANING_OFF 0x00
#define LCD_ZERO_CLEANING_ON 0x01
#define LCD_ZERO_CLEANING_OFF 0x00

//**************************************************


/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_enable_pulse (void);
#define Epulse LCD_enable_pulse
#define enable_pulse_LCD LCD_enable_pulse


/**
 * This function generates the Enable pulse.
 *
 *
 * @param bit_0 bit 0 of the data bus (4 bit modality)
 *
 * @param bit_1 bit 1 of the data bus (4 bit modality)
 *
 * @param bit_2 bit 2 of the data bus (4 bit modality)
 *
 * @param bit_3 bit 3 of the data bus (4 bit modality)
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_send_command (unsigned char bit_3, unsigned char bit_2, unsigned char bit_1, unsigned char bit_0);
#define SendCommand LCD_send_command
#define send_command_LCD LCD_send_command


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void LCD_home(void);
#define HomeLCD LCD_home
#define home_LCD LCD_home


/**
 * This function shifts the LCD screen on the left or right.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_shift(unsigned char shift, unsigned char number_of_shift);
#define ShiftLCD LCD_shift
#define shift_LCD LCD_shift


/**
 * This function shifts the LCD cursor on the left or right. The position of the cursor is where the
 * next writing will be performed.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_shift_cursor(unsigned char shift,unsigned char number_of_shift);
#define ShiftCursorLCD LCD_shift_cursor
#define shift_cursor_LCD LCD_shift_cursor


/**
 * This function locates the LCD cursor on the selected line. 
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Displays. Tested on 20x4 16x2 LCD displays.
 */
void LCD_goto_line (unsigned char line);
#define GotoLineLCD LCD_goto_line
#define goto_line_LCD LCD_goto_line


/**
 * This function locates the LCD cursor to an arbitrary X Y location.
 * Y represents the line number from top.
 *
 * @param x It Specifies horizontal position
 *
 * @param y It Specifies vertical position (line number)
 *
 * @warning It might not work with all the LCD Displays. Tested on 20x4 16x2 LCD displays.
 */
void LCD_goto_xy (unsigned char x, unsigned char y);
#define goto_xy_LCD LCD_goto_xy


/**
 * This function writes a char to the LCD display. (e.g LCD_write_char ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void LCD_write_char (unsigned char value);
#define WriteCharLCD LCD_write_char
#define write_char_LCD LCD_write_char


/**
 * This function writes a const string to the LCD display, e.g LCD_write_message("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write arrays refers the LCD_write_string () function.
 *
 */
#ifndef __XC8
void LCD_write_message(const rom unsigned char *buffer);
#endif

#ifdef __XC8
void LCD_write_message(const unsigned char *buffer);
#endif

#define WriteStringLCD LCD_write_message
#define write_message_LCD LCD_write_message


/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within flash use LCD_write_message ().
 *
 */
void LCD_write_string(unsigned char *buffer);
#define WriteVarLCD LCD_write_string
#define write_string_LCD LCD_write_string


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD display.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-5].
 *                    0: Left Justified
 *                    1-5: Right Justified with n digit
 *
 *  @param zero_cleaning It specifies whether the left zero must be removed [LCD_ZERO_CLEANING_ON, LCD_ZERO_CLEANING_OFF].
 *
 * @note If you set a number of digit less than required the digit will be lost starting from the less
 *       significant digit. Minus is like a digit.
 *
 */
void LCD_write_integer(int value, unsigned char number_of_digits, unsigned char zero_cleaning);
#define WriteIntLCD LCD_write_integer
#define write_integer_LCD LCD_write_integer


/**
 * This Function cleans the LCD display.
 *
 */
void LCD_clear (void);
#define ClearLCD LCD_clear
#define clear_LCD LCD_clear


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [LCD_BLINKING_ON, LCD_BLINKING_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR, LCD_BLINKING_ON, LCD_BLINKING_OFF
 *       to ensure the compatibility with LCD_44780_I2C library.
 *
 */
void LCD_cursor(unsigned char active, unsigned char blinking);
#define CursorLCD LCD_cursor
#define cursor_LCD LCD_cursor


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [LCD_TURN_ON_LED, LCD_TURN_OFF_LED]
 *
 * @param blinking  Let the cursor blink [LCD_BLINKING_ON, LCD_BLINKING_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_LED, LCD_TURN_OFF_LED
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_backlight(unsigned char active);
#define BacklightLCD LCD_backlight
#define backlight_LCD LCD_backlight


/**
 * This function initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  LCD_initialize(unsigned char quartz_frequency);
#define OpenLCD LCD_initialize
#define initialize_LCD LCD_initialize

#endif
