/*******************************************************************************
modulo: 					TXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						17/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ 
							Una stringa composta da 8 caratteri viene inviata al modulo ricevente. Entrambe le stringhe,
							quella trasmessa e quella ricevuta, vengono visualizzate sui display lcd delle schede freedom II
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE
							RE2 = CSN
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)
							
							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//************************************************************
// INCLUDE
//************************************************************
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <pic18f4550.h>
#include "PIC18F4550_config.h"
#include <spi.h>
#include "nrf24l01.h"
#include "LCD_44780.h"
#include "LCD_44780.c"
#include "delay.h"

//************************************************************
// DEFINE 
//************************************************************
#define LINE_SPI_SCK   LATBbits.LATB1         
#define LINE_SPI_SDO   LATCbits.LATC7
#define LINE_SPI_SDI   LATBbits.LATB0
#define LINE_SPI_SS	   LATAbits.LATA5	// non usata

#define LCD_DEFAULT

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************	
int main(void) 
{
	// Variabile utilizzata per il numero delle letture
	unsigned char i;
	
	// Array per i dati da trasmettere
	unsigned char bufferTX[] = "ABCDEFGH";
	
	// Imposto PORTA
	LATA = 0x00;
	TRISA = 0b11011111;		// RA5 impostato in uscita per linea SS
	
	// Imposto PORTB
	LATB = 0x00;
	TRISB = 0b11111101;		// RB0 impostato in ingresso per linea SDI (MISO) - RB1 impostato in uscita per linea SCK (CLOCK)
	
	// Imposto PORTC
	LATC = 0x00;
	TRISC = 0b01111111;		// RC7 impostato in uscita per linea (MOSI)
	
	// Imposto PORTD
	LATD = 0x00;
	TRISD = 0x00;
	
	// Imposto PORTE
	LATE = 0x00;
	TRISE = 0x00;			// RE0 impostato in uscita per linea CE - RE2 impostato in uscita per linea CSN 
		
    // Imposto LCD
    LCD_initialize (20);
	
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;

	// Inizializzo le linee per l'SPI
    LINE_SPI_SCK = 0x00;
    LINE_SPI_SDO = 0x00;
    LINE_SPI_SDI = 0x01;
	
	// Chiudo l'SPI per evitare eventuali errori)
	CloseSPI() ;
  		
	//Inizializzazione del modulo SPI
	//Baudrate Fosc/16 @20MHz -> 1250000 KHz
	OpenSPI(SPI_FOSC_16,MODE_00,SMPEND);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_TX();
	
	while(1)
		{
			// Imposto il display lcd
			LCD_clear ();
			LCD_write_string (bufferTX);
			
			delay_ms(50);
			
					// Pulisco i buffer TX e RX
					Clear_BUFFER_TX_RX ();
										
					// Ripulisco il registro STATUS
					Clear_STATUS_reg ();
				
					// Scrivo il buffer TX
					unsigned char i;
					NRF24L01_CSN = 0;                                    
					WriteSPI(W_TX_PAYLOAD);                       
					for(i = 0; i < PAYLOAD_BYTES; i++)     
					   {
						WriteSPI (bufferTX[i]);
					   }
					NRF24L01_CSN = 1;  
					NRF24L01_CE = 1;
					delay_ms(1);
					NRF24L01_CE = 0;
					delay_ms(100);
		}
}