/***********************************************************************************************
modulo: 					nRF24L01.c (TX - RX)
versione : 					1.4 (Microchip)
autore: 					Marcello Pinna
data:  						28/12/2017

CopyRight 2017 				all rights are reserved
***********************************************************************************************/

//***********************************************************************************************
// INCLUDE
//***********************************************************************************************
#include <xc.h>
#include <stdio.h>
#include <spi.h>
#include "nrf24l01.h"
#include "delay.h"

//***********************************************************************************************
// FUNZIONI PER L'IMPOSTAZIONE DEI MODULI
//***********************************************************************************************
void Setup_CE_CSN (void)
{
	// Imposto le linee CE e CSN in uscita
	NRF24L01_CE = 0;
	NRF24L01_TRIS_CE = 0;
	// Imposto la linea CSN ad 1 prima di iniziare la scrittura sui registri del modulo 
	NRF24L01_CSN = 1;
	NRF24L01_TRIS_CSN = 0;
	delay_ms(100);	
}

void Enable_PWR_UP_CRC (void)
{
	// Abilito PWR_UP e EN_CRC 
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | CONFIG);
	WriteSPI(0x0A);
	NRF24L01_CSN = 1;
	delay_ms(10);
}

void Enable_AUTO_ACK (void)
{
	// Abilito Auto ACK del data pipe 0
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | EN_AA);
	WriteSPI(0x01);
	NRF24L01_CSN = 1;
}

void Enable_DATAPIPE (void)
{
	// Abilito data pipe 0
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | EN_RXADDR);
	WriteSPI(0x01);
	NRF24L01_CSN = 1;
}

void Setup_ADDRESS_WIDTH (void)
{
	// Imposto la larghezza dell'indirizzo a 5 bytes
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | SETUP_AW);
	WriteSPI(0x03);
	NRF24L01_CSN = 1;
}

void Disable_RETRANSMISSION (void)
{
	// Disabilito la ritrasmissione dei dati
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | SETUP_RETR);
	WriteSPI(0x00);
	NRF24L01_CSN = 1;
}

void Setup_CHANNEL (void)
{
	// Imposto il canale 2
	// la frequenza del canale RF sarà uguale a 2402 MHz, secondo la formula F = (2400 + RF_CH) MHz
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | RF_CH);
	WriteSPI(0x02);
	NRF24L01_CSN = 1;
}

void Setup_MPS_DB (void)
{
	// Imposto la velocità di trasmissione ed i decibel (2Mbps, 0db)
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | RF_SETUP);
	WriteSPI(0x0E);
	NRF24L01_CSN = 1;
}

void Setup_WRITING_ADDRESS (void)
{
	// Imposto l'indirizzo di scrittura del data pipe 0
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | TX_ADDR);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	NRF24L01_CSN = 1; 
	NRF24L01_CE = 1;
	delay_ms(1);
	NRF24L01_CE = 0;	
}

void Setup_READING_ADDRESS (void)
{
	// Imposto l'indirizzo di lettura del data pipe 0
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | RX_ADDR_P0);                       
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	WriteSPI(0xE7);
	NRF24L01_CSN = 1; 
	NRF24L01_CE = 1;
	delay_ms(1);
	NRF24L01_CE = 0;
}

void Setup_RX_PAYLOAD (void)
{
	// Imposto il numero dei byte in RX payload nel data pipe 0
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | RX_PW_P0);
	WriteSPI(PAYLOAD_BYTES);
	NRF24L01_CSN = 1;
	delay_ms(100);
}

void Clear_BUFFER_TX_RX (void)
{
	// Pulisco i buffer TX e RX
	NRF24L01_CSN = 0; 
	WriteSPI(FLUSH_TX);
	NRF24L01_CSN = 1; 
	NRF24L01_CSN = 0; 
	WriteSPI(FLUSH_RX);
	NRF24L01_CSN = 1;
}

void Clear_STATUS_reg (void)
{
	// Ripulisco il registro STATUS
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | STATUS);
	WriteSPI(0x70);	
	NRF24L01_CSN = 1;
}

void Setup_TX_Mode (void)
{
	// Impostazione modulo in modalità TX 
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | CONFIG);
	WriteSPI(0x0E);
	NRF24L01_CSN = 1;
	NRF24L01_CE = 0;
	delay_ms(100);
}

void Setup_RX_Mode (void)
{
	// Impostazione modulo in modalità RX 
	NRF24L01_CSN = 0;
	WriteSPI(W_REGISTER | CONFIG);
	WriteSPI(0x0F);
	NRF24L01_CSN = 1;
	NRF24L01_CE = 1;
	delay_ms(100);
}

//***********************************************************************************************
//	FUNZIONE DI CONTROLLO PER RICEZIONE DATI
//***********************************************************************************************
unsigned char NRF24L01_DataReady(void)
{
	unsigned char data;
	NRF24L01_CSN = 0; 
	WriteSPI(R_REGISTER | STATUS);
	data = ReadSPI();
	NRF24L01_CSN = 1; 
	
	if(data == 0x40)
	  {
	   return 0; 	
	  }
	return 1;		
}

//***********************************************************************************************
//  INIZIALIZZAZIONE MODULO TX
//***********************************************************************************************
void NRF24L01_Init_TX (void)
{
	Setup_CE_CSN();
	Enable_PWR_UP_CRC();
	Enable_AUTO_ACK();
	Enable_DATAPIPE();
	Setup_ADDRESS_WIDTH();
	Disable_RETRANSMISSION();
	Setup_CHANNEL();
	Setup_MPS_DB();
	Setup_WRITING_ADDRESS ();
	Setup_READING_ADDRESS ();
	Setup_RX_PAYLOAD();
	Clear_BUFFER_TX_RX();
	Clear_STATUS_reg();
	Setup_TX_Mode();
}

//***********************************************************************************************
//  INIZIALIZZAZIONE MODULO RX
//***********************************************************************************************
void NRF24L01_Init_RX (void)
{
	Setup_CE_CSN();
	Enable_PWR_UP_CRC();
	Enable_AUTO_ACK();
	Enable_DATAPIPE();
	Setup_ADDRESS_WIDTH();
	Disable_RETRANSMISSION();
	Setup_CHANNEL();
	Setup_MPS_DB();
	Setup_WRITING_ADDRESS ();
	Setup_READING_ADDRESS ();
	Setup_RX_PAYLOAD();
	Clear_BUFFER_TX_RX();
	Clear_STATUS_reg();
	Setup_RX_Mode();
}

