/*******************************************************************************
modulo: 					TXmain.c
versione : 					1.0
autore: 					Massimo Peino
data:  						23/02/2018
descrizione: 				        Trasmissione di un byte con l'utilizzo  del modulo NFR24L01+
						Il byte trasmesso viene visualizzato da 8 led collegati alla PORTD
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)


compilatore:			         	mikroC PRO for PIC v.7.1.0

Configurazione hardware:
							RE0 = CE			(si puo' usare qualunque altro pin)
							RE2 = CSN			(si puo' usare qualunque altro pin)
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)


CopyRight 2018 				all rights are reserved
*******************************************************************************/



//***************************************************************************
//  DEFINE
//***************************************************************************
#define NRF_CE          LATE.B0
#define NRF_TRIS_CE     TRISE.B0

#define NRF_CSN         LATE.B2
#define NRF_TRIS_CSN    TRISE.B2

#define SPI_SCK         LATB.B1
#define SPI_SDO         LATC.B7
#define SPI_SDI         LATB.B0

// Numero di byte da inviare o leggere
#define PAYLOAD          1

//***************************************************************************
//  MAPPA DEL REGISTRO 
//***************************************************************************
//                Mnemonic                 value
#define R_REGISTER       0X00
#define W_REGISTER       0X20
#define R_RX_PAYLOAD     0X61
#define W_TX_PAYLOAD     0XA0
#define FLUSH_TX         0XE1
#define FLUSH_RX         0XE2
#define REUSE_TX_PL      0XE3

#define CONFIG           0X00
#define EN_AA            0X01
#define EN_RXADDR        0X02
#define SETUP_AW         0X03
#define SETUP_RETR       0X04
#define RF_CH            0X05
#define RF_SETUP         0X06
#define STATUS           0X07
#define OBSERVE_TX       0X08
#define CD               0X09
#define RX_ADDR_P0       0X0A
#define RX_ADDR_P1       0X0B
#define RX_ADDR_P2       0X0C
#define RX_ADDR_P3       0X0D
#define RX_ADDR_P4       0X0E
#define RX_ADDR_P5       0X0F
#define TX_ADDR          0X10
#define RX_PW_P0         0X11
#define RX_PW_P1         0X12
#define RX_PW_P2         0X13
#define RX_PW_P3         0X14
#define RX_PW_P4         0X15
#define RX_PW_P5         0X16
#define FIFO_STATUS      0X17

//***************************************************************************
//  PROTOTIPI DI FUNZIONE
//***************************************************************************
void SPI1_Close(void);
void NRF24L01_Init(void);

//***************************************************************************
//  FUNZIONE PER CHIUSURA SPI
//***************************************************************************
void SPI1_Close(void)
{
SSPCON1 &= 0XDF;
}

//***********************************************************************************************
// FUNZIONI PER L'IMPOSTAZIONE DEI MODULI
//***********************************************************************************************
void NRF24L01_Init(void)
{
                       
                       // Imposto le linee CE e CSN in uscita
        NRF_CE =0;
        NRF_TRIS_CE =0;
                
                     // Imposto la linea CSN ad 1 prima di iniziare la scrittura sui registri del modulo
        NRF_CSN = 1;
        NRF_TRIS_CSN = 0;
        delay_ms (100);

                    // Abilito PWR_UP e EN_CRC
        NRF_CSN = 0;
        SPI1_Write (W_REGISTER | CONFIG ) ;
        SPI1_Write  (0X0A);
        NRF_CSN = 1;
        Delay_ms (10);
                
                   // Abilito Auto ACK del data pipe 0
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | EN_AA ) ;
        SPI1_Write (0x01);
        NRF_CSN = 1;
                
                  // Abilito data pipe 0
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | EN_RXADDR ) ;
        SPI1_Write (0x01);
        NRF_CSN = 1;
                
                 // Imposto la larghezza dell'indirizzo a 5 bytes
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | SETUP_AW ) ;
        SPI1_Write (0x03);
        NRF_CSN = 1;
                
                // Disabilito la ritrasmissione dati
        NRF_CSN = 0;
        SPI1_Write (W_REGISTER | SETUP_RETR);
        SPI1_Write (0x00);
        NRF_CSN = 1;

                // Imposto il canale 2
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | RF_CH ) ;
        SPI1_Write (0x02);
        NRF_CSN = 1;
                
                // Imposto la velocità di trasmissione ed i decibel (2Mbps, 0db)
        NRF_CSN = 0;
        SPI1_Write (W_REGISTER | RF_SETUP);
        SPI1_Write (0x0E);
        NRF_CSN = 1;

                // Imposto l'indirizzo di scrittura del data pipe 0
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | TX_ADDR ) ;
        SPI1_Write (0x05);
        SPI1_Write (0x04);
        SPI1_Write (0x03);
        SPI1_Write (0x02);
        SPI1_Write (0x01);
        NRF_CSN = 1;
        NRF_CE =1;
        delay_ms(1);
        NRF_CE = 0;
                
                // Imposto l'indirizzo di lettura del data pipe 0
        NRF_CSN = 0;
        SPI1_Write(W_REGISTER | RX_ADDR_P0);
        SPI1_Write (0x05);
        SPI1_Write (0x04);
        SPI1_Write (0x03);
        SPI1_Write (0x02);
        SPI1_Write (0x01);
        NRF_CSN = 1;
        NRF_CE = 1;
        delay_ms(1);
        NRF_CE = 0;

                // Imposto il numero dei byte in RX payload nel data pipe 0
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | RX_PW_P0 ) ;
        SPI1_Write (PAYLOAD);
        NRF_CSN = 1;

                // Pulisco i buffer TX e RX
        NRF_CSN =0;
        SPI1_Write  (FLUSH_TX ) ;
        NRF_CSN = 1;
        NRF_CSN = 0;
        SPI1_Write (FLUSH_RX);
        NRF_CSN = 1;

                // Ripulisco il registro STATUS
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | STATUS ) ;
        SPI1_Write (0X70);
        NRF_CSN = 1;

                // Impostazione modulo in modalità TX 
        NRF_CSN =0;
        SPI1_Write  (W_REGISTER | CONFIG ) ;
        SPI1_Write (0X0E);
        NRF_CSN = 1;
        NRF_CE = 0;
        delay_ms(100);
}

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************        
void main()
{
        // Array per memorizzare i dati da trasmettere
                unsigned char bufferTX;
                
                // Imposto PORTA
        LATA = 0x00;
        TRISA = 0b11011111;
                
                // Imposto PORTB
        LATB = 0X00;
        TRISB = 0b11111101;
                
                // Imposto PORTC
        LATC = 0X00;
        TRISC = 0b01111111;
                
                // Imposto PORTD
        LATD = 0X00;
        TRISD = 0X00;
                
                // Imposto PORTE
        LATE = 0X00;
        TRISE = 0X00;
                
                // Imposto analogici PORTA in digitale
        ADCON1 = 0X0F;
                
                // Imposto oscillatore interno
        OSCCON = 0b01110010;
                
                // Inizializzo le linee per l'SPI
        SPI_SCK = 0X00;
        SPI_SDO = 0X00;
        SPI_SDI = 0X01;
                
                // Chiudo l'SPI per evitare eventuali errori
                SPI1_Close();
                
                //Inizializzazione del modulo SPI
        SPI1_Init_Advanced (_SPI_MASTER_OSC_DIV16, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
                
                // Inizializzo il modulo NFR24L01 in modalità trasmissione
        NRF24L01_Init() ;
  
        while(1)
          {
                // byte da trasmettere
                                bufferTX  = 15;
          
                                // Pulisco i buffer TX e RX
                NRF_CSN =0;
                SPI1_Write  (FLUSH_TX ) ;
                NRF_CSN = 1;

                NRF_CSN =0;
                SPI1_Write  (FLUSH_RX ) ;
                NRF_CSN = 1;
                                
                                // Ripulisco il registro STATUS
                NRF_CSN =0;
                SPI1_Write  (W_REGISTER | STATUS ) ;
                SPI1_Write (0X70);
                NRF_CSN = 1;
                                
                                // Scrivo il buffer TX
                NRF_CSN =0;
                SPI1_Write  (W_TX_PAYLOAD ) ;
                SPI1_Write (bufferTX);
                NRF_CSN = 1;

                NRF_CE =1;
                delay_ms(1);
                NRF_CE = 0;
                                
                                // Visualizzo con i led il dato trasmesso
                LATD = bufferTX;
          }

}